//============================================================================
// Name        : api_test.cpp
// Author      : RiBiG
// Version     :
// Copyright   : Copyright 2019
// Description : Hello World in C++, Ansi-style
//============================================================================

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <fcntl.h>
#include <pthread.h>

#include "mxnet2.h"

#define LOWORD(l)           ((short)(l & 0xffff))
#define HIWORD(l)           ((short)((l >> 16) & 0xffff))

// Set your own USERCODE
//#define USERCODE	 1428497  //34567
#define USERCODE	 1234  //34567

// How many threads to start
#define NO_THREADS	4

// How many times testFunc() is to be run in a thread
#define LOOP_COUNT	50

// AppSlot for LogIn_MatrixNet
// Set a license count in this AppSlot of your test dongle.
#define TARGET_APPSLOT	3

_mxINT16 dngPort = 85;

int testFunc( _mxINT32 UserCode, _mxINT16 dNr, int threadNo )
{
	int result;
	_mxINT32 data[16];
	_mxINT32 version;


	// Client API Version
	version = rGetVersionAPI();
	printf("%d:API version=%d\n", threadNo, version);

	// Server Version
	version = rDongle_Version(0, 85);
	if (version > 0)
		printf("%d:Server Version =%d\n", threadNo, version);
	else
		printf("%d:Server Version Error: %d\n", threadNo,version);

	//
	// Time at which the server detected the dongles
	//
	_mxINT32 detectTime;
	_mxINT32 timeBuf[2];

	// the server returns Unix time in time_t which is __int64
	// timeBuf[0] receives lower 32bit and timeBuf[1]
	// receives upper 32bit.  timeBuf[0] will overflow in 2038.
	// thus timeBuf[1] will be 0 till then
	detectTime = rDongle_GetDetectTime(UserCode, timeBuf, dNr, 85);
	if (detectTime > 0)
		printf("%d:Dongle Detect Time =%u %u\n", threadNo, timeBuf[0], timeBuf[1]);
	else
		printf("%d:Dongle_GetDetectTime Error %d\n", threadNo, detectTime);


	version = rDongle_Version(dNr, 85);
	if( version > 0 )
		printf("%d:Dongle Version =%d.%d\n", threadNo, HIWORD(version), LOWORD(version));
	else
		printf("%d:Dongle_Version Error: %d\n", threadNo, version);

	// Read & Write
	result = rDongle_ReadData( UserCode,data, 5, dNr, 85 );
	if ( result == 5 )
		printf("%d:ReadData --> %u %u %u %u %u\n", threadNo, data[0], data[1], data[2], data[3], data[4]);
	else
		printf("%d:ReadData Error %d\n", threadNo, result);

	result = rDongle_ReadDataEx(UserCode, data, 15, 1, dNr, 85);
	if (result == 1)
		printf("%d:ReadDataEx %u \n", threadNo, data[0]);
	else
		printf("%d:ReadDataEx Error %d \n", threadNo, result);

	result = rDongle_GetKeyFlag( UserCode, dNr,85 );

	if( result == 0 )
	{
		_mxINT32 keydata[4];
		keydata[0]=49038035;
		keydata[1]=740392783;
		keydata[2]=1294833;
		keydata[3]=93803484;

		// write API always fails
		result = rDongle_WriteKey(UserCode, keydata, dNr, 85 );
		if( result > 0 )
			printf("enckey written\n");
		else
			printf("%d:enckey write error %d\n", threadNo, result);
	}
	else
	{
		printf("GetKeyFlag : %d\n", result);
	}

	// Login
	bool bLogin=false;

	result = rLogIn_MatrixNet(UserCode, TARGET_APPSLOT, dNr);
	if (result < 0)
		printf("%d:Login Error %d\n", threadNo, result);
	else
	{
		printf("%d:LogIn dNr=%d appSlot=%d result=%d\n", threadNo, dNr, TARGET_APPSLOT, result);
		bLogin = true;
	}

	// Get Serial Number
	long ser;
	ser = rDongle_ReadSerNr(UserCode, dNr,85);
	if ( ser < 0 )
		printf( "%d:SerNo Error %ld\n", threadNo, ser);
	else
		printf( "%d:SerNo: %ld\n", threadNo, ser);

	result = rDongle_ReadDataEx( UserCode,data, 7, 5, dNr, 85 );
	if ( result < 0 )
		printf("data read error %d\n", result );
	else
	{
		printf( "%d:ReadDataEx Count=%d\n", threadNo, result);
		printf( "%d:data %u %u %u %u %u\n", threadNo, data[0], data[1], data[2], data[3], data[4]);

		data[0]++;
		data[1]++;
		data[2]++;
		data[3]++;
		data[4]++;

		// write API always fails
		result = rDongle_WriteDataEx(UserCode, data, 7, 5, dNr, 85);
		if( result < 0 )
			printf("data write error %d\n", result );
		else
			printf( "data write Count=%d\n", result);
	}


	// Encrypt
	data[0]=12345;
	data[1]=56789;

	result = rDongle_EncryptData(UserCode,data, dNr,85);
	if( result > 0 )
		printf( "%d:12345 56789 ->data encrypted %u %u\n", threadNo, data[0], data[1]);
	else
		printf("%d:encrypt error %d\n", threadNo, result );

	// Decrypt to see if the original data are returned
	result = rDongle_DecryptData(UserCode,data, dNr, 85);
	if( result > 0 )
		printf( "%d:data decrypted -> %u %u\n", threadNo, data[0], data[1]);
	else
		printf("%d:decrypt error %d\n", threadNo, result );

	//
	// Query client names that have licenses
	//

	short nLoginCount = (short)rDongle_GetLoginCount(UserCode, TARGET_APPSLOT, (short)dNr, (short)85);
	printf("%d:Login Count=%d\n", threadNo, nLoginCount);

	char *pBuf = NULL;

printf("will call GetLoginCountEx\n");
	
	nLoginCount = (short)rDongle_GetLoginCountEx(UserCode, TARGET_APPSLOT, (short)dNr, (short)85, &pBuf);
	printf("%d:Login CountEx=%d\n", threadNo, nLoginCount);
	if (nLoginCount > 0 && pBuf != NULL)
		printf("%s\n", pBuf);

	if (pBuf != NULL)
	{
printf("will free pBuf\n");
		free(pBuf);
	}
//	getchar();

	// Logout only when logged in
	if (bLogin == true)
	{
		result = rLogOut_MatrixNet(UserCode, TARGET_APPSLOT, dNr);
		if (result < 0)
			printf("%d:Logout error %d\n", threadNo, result);
		else
			printf("%d:LogOut dNr=%d appSlot=%d result=%d\n", threadNo, dNr, TARGET_APPSLOT, result);
	}
	else
		printf("Not logged-in\n");

	return 0;
}


void *test(void* pParam)
{
	int  threadNo = *(int*)pParam;

	printf("thread no %d\n", threadNo);

	// repeat LOOP_COUNT times
	int n;
	for (n = 0; n < LOOP_COUNT; n++)
	{


		int retry = 0;

		do
		{
			int result = rInit_MatrixAPI();

			if (result < 0)
			{
				sleep(3);
				retry++;
			}
			else
				break;

		} while (retry < 5);

		if (retry == 5)
		{
			printf("rInit_MatrixAPI failed\n");
			sleep(3);
			continue;
		}

		printf("%d:rInit_MatrixAPI success\n", threadNo);

		int nCount  = rDongle_Count(85);
		if (nCount  < 0)
		{
			printf("%d:Count Error %d\n", threadNo,nCount);
			sleep(3);
			continue;
		}
		else
			printf("%d:Count %d No=%d\n", threadNo, nCount, threadNo);


		// run the test function
		for (int dNr = 1; dNr <= nCount; dNr++)
		{
			testFunc(USERCODE, dNr, threadNo);
		}

		printf("%d:rRelease_MatrixAPI = %d\n", threadNo, rRelease_MatrixAPI());
	}

	return 0;
}


int main(int argc, char* argv[])
{
	pthread_t thread[NO_THREADS];

	for (int i = 0; i < NO_THREADS; i++)
	{
		int ret = pthread_create( &thread[i], NULL, test, (void*)&i);
		if( ret )
		{
			 fprintf(stderr,"pthread_create Error: %d\n", ret);
	         exit(EXIT_FAILURE);

		}
		usleep(100000);
	}

	for (int i = 0; i < NO_THREADS; i++)
		pthread_join( thread[i], NULL);
}
