//
//  RemoteLicenseMgr.swift
//  remoteAPITest
//
//  Copyright © 2019 RiBiG Inc. All rights reserved.
//

import Foundation

class RemoteLicenseMgr{
    
    enum MethodCall{
        case Acquire
        case Release
        case Timer
    }
    
    enum APICall {
        case Init( Int16 )
        case LogIn( Int16 )
        case LogOut( Int16 )
        case Release( Int16 )
    }

    var initAPICount = 0
    let dispatchQueue = DispatchQueue(label: "jp.co.ribig.mxnet2api")  // serial queue
    let timer = RepeatingTimer( timeInterval: 120)
    
    var notificationHandler : (MethodCall, APICall) -> Void
    
    static func makeCString(from str: String) -> UnsafeMutablePointer<Int8> {
        let count = str.utf8CString.count
        let result: UnsafeMutableBufferPointer<Int8> = UnsafeMutableBufferPointer<Int8>.allocate(capacity: count)
        _ = result.initialize(from: str.utf8CString)
        return result.baseAddress!
    }
    
    init (ip : String, port : Int, timeout : Int, notificationHandler : @escaping (MethodCall, APICall) -> Void )
    {
        // Setup the server IP/Port
        MxNet_SetIP( RemoteLicenseMgr.makeCString( from: ip ))
        MxNet_SetPort( Int32(port) )
        MxNet_SetTimeOut( Int32(timeout) )
        
        self.notificationHandler = notificationHandler
        
        // handler to update periodically the appSlot in the background
        // once LogIn_MatrixNet is successful
        timer.eventHandler = {
            print("timer")
            
            self.dispatchQueue.async{
                
                var ret = rLogIn_MatrixNet(1234, 3, 1)
                if ret < 0 {
                    // -105: the server is running but the session is not found
                    // try to create a new session
                    if ret == -105{
                        rReset_MatrixAPI()
                        self.initAPICount = 0
                        
                        ret = rInit_MatrixAPI()
                        if ret >= 0 {
                            self.initAPICount += 1
                            ret = rLogIn_MatrixNet(1234, 3, 1)
                            
                            if ret < 0 { self.notificationHandler(.Timer, .LogIn(ret))}
                        }
                        else{
                            self.notificationHandler(.Timer, .Init(ret))
                        }
                    }
                    else{
                        self.notificationHandler(.Timer, .LogIn(ret))
                    }
                    
                }
                
            } // self.dispatch.async
            
        } // timer.eventHandler
    }
    
    // we want to make sure that the open session is closed
    func CloseConnections(){
       
       while( initAPICount > 0 ){
          rRelease_MatrixAPI()
          initAPICount -= 1
        }
    }
    
    func SetNotificationHandler( handler : @escaping (MethodCall,APICall) -> Void){
            notificationHandler = handler
    }
    
    func GetDispatchQueue() -> DispatchQueue{
        return dispatchQueue
    }
    
    func AcquireLicense()
    {
        dispatchQueue.async
        {
            var ret = rInit_MatrixAPI()
            
            guard ret >= 0 else {
                self.notificationHandler( .Acquire, .Init(ret))
                return
            }
            
            ret = rLogIn_MatrixNet(1234, 3, 1)
            
            guard ret >= 0 else{
                rRelease_MatrixAPI()
                self.notificationHandler( .Acquire , .LogIn(ret))
                return
            }
            
            self.initAPICount += 1
            self.timer.resume()
            
            self.notificationHandler( .Acquire, .LogIn(ret))
        }
    }

    func ReleaseLicense()
    {
        if initAPICount == 0 { return }
        
        //let queue = DispatchQueue.main
        dispatchQueue.async {
            
            var ret = rLogOut_MatrixNet(1234,3,1)
            
            // LogOut successful
            if( ret >= 0 ){
                self.timer.suspend()
            }
            else{
                self.notificationHandler( .Release, .LogOut(ret))
            }
            
            ret = rRelease_MatrixAPI()

            if ret < 0 {
                self.notificationHandler( .Release, .Release(ret))
            }
            else{
                self.initAPICount -= 1
            }
            
        } // dispatchQueue.async
    }
    
    
}
